//------------------------------------------------------------------------------
//--  Input Capture - GPIO Support                                            --
//--                                                                          --
//--  Copyright (c) 2006 by Eric Fahlgren & Lance Gardiner                    --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#include "hcs12def.h"
#include "ms2def.h"
#include "adc.h"

#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#  pragma DATA_SEG  DEFAULT
#  pragma CONST_SEG DEFAULT
#endif

//------------------------------------------------------------------------------

static s_ADC adc[N_ADCS];

void adc_init(void)
{
   UINT08 iConversions;

   UINT08 iChannel;
   for (iChannel = 0; iChannel < N_ADCS; iChannel++) {
      adc[iChannel].enabled = FALSE;
   }

   //---------------------------------------------------------------------------
   //--  Set up ADCs so they continuously convert, then read result registers
   //--  every millisecond.
   ATD0CTL2  = 0x40;  // Leave interrupt disabled, set fast flag clear.
   ATD0CTL3  = 0x00;  // Do 8 conversions/ sequence.
   ATD0CTL4  = 0x67;  // 10-bit resoln, 16 tic cnvsn (max accuracy),
                      // Prescaler divide by 16 => 2/3 us tic x 18 tix.
   ATD0CTL5  = 0xB0;  // Right justified, unsigned, continuous cnvsn,
                      // sample 8 channels starting with AN0.
   ATD0CTL2 |= 0x80;  // Turn on ADC0.

   //---------------------------------------------------------------------------
   //--  Wait for ADC engine charge-up or P/S ramp-up.
   for (iConversions = 0; iConversions < 160; iConversions++)  {
      while (!(ATD0STAT0 >> 7)) ; // wait til conversion complete
      ATD0STAT0 = 0x80;
   }
}

//------------------------------------------------------------------------------

void adc_enable(UINT08 index, SINT16 *r)
{
   adc[index].enabled = TRUE;
   adc[index].result  = r;
}

//------------------------------------------------------------------------------

void adc_get(UINT08 iChannel)
{
   *adc[iChannel].result = ATD0DR[iChannel];
}

//------------------------------------------------------------------------------

void adc_update(void)
{
   UINT08 iChannel;
   for (iChannel = 0; iChannel < N_ADCS; iChannel++) {
      if (adc[iChannel].enabled) adc_get(iChannel);
   }
}

//------------------------------------------------------------------------------
