/*************************************************************************
**************************************************************************
**   M E G A S Q U I R T  II - 2 0 0 4 - V1.000
**
**   (C) 2003 - B. A. Bowling And A. C. Grippo
**
**   This header must appear on all derivatives of this code.
**
***************************************************************************
**************************************************************************/

/*************************************************************************
**************************************************************************
**   GCC Port
**
**   (C) 2004,2005 - Philip L Johnson
**
**   This header must appear on all derivatives of this code.
**
***************************************************************************
**************************************************************************/

//------------------------------------------------------------------------------
//--  CAN IO - I/O for CAN-Enabled MegaSquirt                                 --
//--                                                                          --
//--  Copyright (c) 2005 by Eric Fahlgren & Lance Gardiner                    --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------
//--  Version 1.000                                                           --
//--     EF 2005-10-02 - Derived from Version 2.300 of the base MS-II code.   --
//--                                                                          --
//--                                                                          --
//--                                                                          --
//------------------------------------------------------------------------------

//==============================================================================
//== CAN subsystem  ============================================================

#include "hcs12def.h"
#include "can_io.h"

#define  CAN_NMSG          10

#define  MSG_CMD            0
#define  MSG_REQ            1
#define  MSG_RSP            2
#define  MSG_XSUB           3
#define  NO_VAR_BLKS       16

typedef struct {
   UINT32  sendClock;

   UINT08 *Data[NO_VAR_BLKS];
   UINT16  Status;
   UINT08  doReset;
   UINT08  doSend;
   UINT08  ID;
} canObject;

canObject cans; // Singleton

typedef struct {
   UINT08  type;
   UINT08  varblk;
   UINT08  dest;
   UINT08  varbyt;
   UINT16  varoff;
   UINT08  datbuf[8];  // max msg data = 8 bytes
} canMsg;

typedef struct {
   // CAN transmit message ring buffer:
   //    can[0] is to hold Rx,TxISR messages,
   //    can[1] for main loop messages
   UINT08  no;
   UINT08  in;
   UINT08  out;
   canMsg  msg[CAN_NMSG];
} canBuffer;

canBuffer can[2];

//------------------------------------------------------------------------------

void Can_Init(UINT08 thisBoardId)
{
   UINT08 ix;

//   cans.Data[0] = (UINT08 *) &ramInputs;
//   cans.Data[1] = (UINT08 *) &ramOutputs;
   for (ix = 2; ix < NO_VAR_BLKS; ix++) {             // rest spares for now
      cans.Data[ix] = 0;
   }

   cans.doReset = FALSE;
   cans.ID      = thisBoardId;   // GPIO is processor 1
   cans.doSend  = TRUE;

   cans.sendClock = MS1000;

   /* Set up CAN communications */
   /* Enable CAN, set Init mode so can change registers */
   CANCTL1 |= 0x80;
   CANCTL0 |= 0x01;

   /* clear ring buffers */
   for (ix = 0; ix < 2; ix++) {
      can[ix].no     = 0;
      can[ix].in  = 0;
      can[ix].out = 0;
   }

   while (!(CANCTL1 & 0x01)) /*empty*/;     // make sure in init mode

   /* Set Can enable, use IPBusclk (24 MHz),clear rest */
   CANCTL1 = 0xC0;
   /* Set timing for .5Mbits/ sec */
   CANBTR0 = 0xC2; /* SJW=4,BR Prescaler= 3(24MHz CAN clk) */
   CANBTR1 = 0x1C;                    /* Set time quanta: tseg2 =2,tseg1=13
                                            (16 Tq total including sync seg (=1)) */
   CANIDAC = 0x00; /* 2 32-bit acceptance filters */
   /* CAN message format:
      Reg Bits: 7 <-------------------- 0
      IDR0:    |---var_off(11 bits)----|  (Header bits 28 <-- 21)
      IDR1:    |cont'd 1 1 --msg type--|  (Header bits 20 <-- 15)
      IDR2:    |---From ID--|--To ID---|  (Header bits 14 <--  7)
      IDR3:    |--var_blk-|--spare--rtr|  (Header bits  6 <-- 0,rtr)
    */
   /* Set identifier acceptance and mask registers to accept
      messages only for canID or device #15 (=> all devices) */
   /* 1st 32-bit filter bank-to mask filtering, set bit=1 */
   CANIDMR0 = 0xFF;                       // anything ok in IDR0(var offset)
   CANIDAR1 = 0x18;                                         // 0,0,0,SRR=IDE=1
   CANIDMR1 = 0xE7;                 // anything ok for var_off cont'd, msgtype
   CANIDAR2 = cans.ID;                          // rcv msg must be to canID, but
   CANIDMR2 = 0xF0;                            // can be from any other device
   CANIDMR3 = 0xFF;                                 // any var_blk, spare, rtr
   /* 2nd 32-bit filter bank */
   CANIDMR4 = 0xFF;                       // anything ok in IDR0(var offset)
   CANIDAR5 = 0x18;                                         // 0,0,0,SRR=IDE=1
   CANIDMR5 = 0xE7;                 // anything ok for var_off cont'd, msgtype
   CANIDAR6 = 0x0F;                 // rcv msg can be to everyone (id=15), and
   CANIDMR6 = 0xF0;                            // can be from any other device
   CANIDMR7 = 0xFF;                                 // any var_blk, spare, rtr

   /* clear init mode */
   CANCTL0 &= 0xFE;
   /* wait for synch to bus */
   while (!(CANCTL0 & 0x10));

   /* no xmit yet */
   CANTIER = 0x00;
   /* clear RX flag to ready for CAN recv interrupt */
   CANRFLG = 0xC3;
   /* set CAN rcv full interrupt bit */
   CANRIER = 0x01;

   cans.doReset = FALSE;
}

//------------------------------------------------------------------------------

UINT08 Can_NeedsReset()
{
   return cans.doReset;
}

//------------------------------------------------------------------------------

extern UINT32 tix;

void Can_Send()
{
   if (tix > cans.sendClock) {
      UINT08 ix;
      cans.sendClock = tix + MS1000;              // 1 sec(7812 x .128 ms) clk
      for (ix = 0; ix < 6; ix++) {
         // load ring buffer - send rpm
//         (void) memcpy(&can[1].msg[can[1].in].datbuf[0], (char *) &ramOutputs.rpm, 2);
         can[1].msg[can[1].in].type = MSG_CMD;
         can[1].msg[can[1].in].varblk = 1;

         // Told to put rpm in ramOutputs.spare[3]; below is offset
//         can[1].msg[can[1].in].varoff = (unsigned short) (&ramOutputs.spare[3]) -
//         (unsigned short) (&ramOutputs);
         can[1].msg[can[1].in].dest = 1;          // send to device 1
         can[1].msg[can[1].in].varbyt = 2;                   // 2 bytes

         // This is where (in xmt ring buffer) to put next message
         if (can[1].in < CAN_NMSG-1)
            can[1].in++;
         else
            can[1].in = 0;

         // increment counter
         if (can[1].no < CAN_NMSG)
            can[1].no++;
         else
            can[1].no = CAN_NMSG;
      }

      if (!(CANTIER & 0x07)) {
         // Following will cause entry to TxIsr without sending msg
         // since when CANTIER = 0, CANTFLG left as buff empty(>0).
         // If CANTIER has at least 1 int buf enabled, will enter
         // TxIsr automatically.
         CANTBSEL = CANTFLG;
         CANTIER  = CANTBSEL;
      }
   }
}

//------------------------------------------------------------------------------

#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif
INTERRUPT_FUNC void CanTxIsr()
{
   UINT08 ix;

   /* CAN Xmit Interrupt */
   CANTBSEL = CANTFLG;                           // select MSCAN xmit buffer

   // Check ring buffers and xfer to MSCAN buffer
   for (ix = 0; ix < 2; ix++) {
      if (can[ix].no) {
         /* Set up identifier registers */
         CAN_TB0_IDR0 = (UINT08) (can[ix].msg[can[ix].out].varoff >> 3);
         // 8 high bits in IDR0, 3 low bits in IDR1
         CAN_TB0_IDR1 = (UINT08) ((can[ix].msg[can[ix].out].varoff & 0x0007) << 5)
                      | 0x18                                    // SRR=IDE=1
                      | can[ix].msg[can[ix].out].type;  // 3 bits
         CAN_TB0_IDR2 = (cans.ID << 4) | can[ix].msg[can[ix].out].dest;
         CAN_TB0_IDR3 = (can[ix].msg[can[ix].out].varblk << 4);
         CAN_TB0_DLR  = can[ix].msg[can[ix].out].varbyt;

         /* Set xmt buffer priorities (lower is > priority) */
         CAN_TB0_TBPR = 0x02;

         /* set data in buffer */
         switch (can[ix].msg[can[ix].out].type) {
            case MSG_CMD:  // msg for dest ecu to set a variable to val in msg
            case MSG_RSP:  // msg in reply to dest ecu's request for a var val
//???          memcpy((char *)&(CAN_TB0_DSR0), (char *)&can[ix].msg[can[ix].out].datbuf[0], CAN_TB0_DLR);
               break;

            case MSG_REQ:  // msg to send back current value of variable(s)
               CAN_TB0_IDR3 |= 0x01; //rtr=1 (remote frame,no data-ignores DLR)
               break;

            case MSG_XSUB:
               CAN_TB0_DLR = 0;
               break;
         }

         // This is where (in xmt buffer) to get next outgoing message
         if (can[ix].out < (CAN_NMSG - 1))
            can[ix].out++;
         else
            can[ix].out = 0;
         if (can[ix].no > 0)
            can[ix].no--;  // TB buf loaded for xmit - decrement ring buf count
         cans.Status &= CLR_XMT_ERR;
         /* set CAN xmt interrupt bit and clear flag to initiate transmit */
         CANTIER |= CANTBSEL;
         CANTFLG = CANTBSEL;          // 1 clears(buffer full), 0 is ignored
         break;                          // only handle 1 buffer entry at time
      }
      if (ix == 1) {                   // nothing left in either ring buffer
         CANTIER = 0x00;  // leave CANTFLG as buffer empty, but disable interrupt
         break;
      }
   }

   if ((CANRFLG & ~0x0C) != 0) {
      // Xmt error count
      cans.Status |= XMT_ERR;
      //reset = 1;
   }
}
#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#endif

//------------------------------------------------------------------------------

#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif
INTERRUPT_FUNC void CanRxIsr()
{
      UINT08  rcv_id, msg_type, var_blk, var_byt;
      unsigned short var_off;


   /* CAN Recv Interrupt */
      if (CANRFLG & 0x01) {

         var_off = ((unsigned short) CAN_RB_IDR0 << 3) |
         ((CAN_RB_IDR1 & 0xE0) >> 5);
         msg_type = (CAN_RB_IDR1 & 0x07);
         rcv_id = (CAN_RB_IDR2 & 0xF0) >> 4;
         var_blk = (CAN_RB_IDR3 & 0xF0) >> 4;
         var_byt = (CAN_RB_DLR & 0x0F);

         switch (msg_type) {

         case MSG_CMD:     // msg for this ecu to set a variable to val in msg
         case MSG_RSP:     // msg in reply to this ecu's request for a var val
            // value in the data buffer in recvd msg
            switch (rcv_id) {
               case 0:                                // message from device 0
               case 1:                                // message from device 1
                  switch (var_blk) {
                     case 0:
                     case 1:
                        // update variable value with received data
//???                   (void)memcpy((char *)cans.Data[var_blk] + var_off, (char *)(&CAN_RB_DSR0), var_byt);
                        break;
                  }
                  break;
            }
               break;

         case MSG_REQ:        // msg to send back current value of variable(s)
            switch (rcv_id) {
               case 0:                                // message from device 0
               case 1:                                // message from device 1
                  switch (var_blk) {
                     case 0:
                     case 1:
                        // put variable value(s) in xmit ring buffer
//???                           (void) memcpy((char *)&can[0].msg[can[0].in].datbuf[0],
     //???                                    (char *)(cans.Data[var_blk] + var_off), var_byt);
                           break;
                  }
                     break;
            }

            // Update related parameters for xmt ring buffer
               can[0].msg[can[0].in].type = MSG_RSP;
               can[0].msg[can[0].in].varblk = var_blk;
               can[0].msg[can[0].in].varoff = var_off;
               can[0].msg[can[0].in].dest = rcv_id;
               can[0].msg[can[0].in].varbyt = var_byt;
            // This is where (in xmt buffer) to put next messge
               if (can[0].in < (CAN_NMSG - 1))
                  can[0].in++;
               else
                  can[0].in = 0;
            // increment counter
               if (can[0].no < CAN_NMSG)
                  can[0].no++;
               else
                  can[0].no = CAN_NMSG;
               if (!(CANTIER & 0x07)) {
                  // Following will cause entry to TxIsr without sending msg
                  // since when CANTIER = 0, CANTFLG left as buff empty.
                  // If CANTIER has at least 1 int buf enabled, will enter
                  // TxIsr automatically.
                  CANTBSEL = CANTFLG;
                  CANTIER  = CANTBSEL;
               }
               break;

         case MSG_XSUB:              // msg to execute a subroutine
            switch (rcv_id) {
               case CAN_MS2_DEVICE:  // message to this ecu from device 1
//                Can_MS2Send();      // execute sub immediately here (set
                                     // flag if can execute in main loop)
                   break;
            }
               break;
      }                              // end msg_type switch
       cans.Status &= CLR_RCV_ERR;
   }
      if ((CANRFLG & ~0x72) != 0) {
      // Rcv error or overrun on receive
         cans.Status |= RCV_ERR;
      //reset = 1;
   }
   /* clear RX buf full, err flags to ready for next rcv int */
   /*  (Note: can't clear err count bits) */
      CANRFLG = 0xC3;
}
#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#endif

//==============================================================================
