//------------------------------------------------------------------------------
//--  Digital IO - GPIO Support                                               --
//--                                                                          --
//--  Copyright (c) 2006 by Eric Fahlgren & Lance Gardiner                    --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#include "hcs12def.h"
#include "ms2def.h"
#include "gpio.h"

#ifndef GCC_BUILD
#  pragma CODE_SEG DEFAULT
#endif

//------------------------------------------------------------------------------

static s_digIO digIO[N_PORTS];

//------------------------------------------------------------------------------

void digIO_init(void)
{
   UINT08 iPort;
   for (iPort = 0; iPort < N_PORTS; iPort++) {
      digIO[iPort].state = io_OFF;
   }
}

//------------------------------------------------------------------------------

void digInp_enable(UINT08 iPort, SINT16 *result)
{
   if (0 >= iPort && iPort <= 7) ATD0DIEN |= portInfo[iPort].mask;
   *portInfo[iPort].ddrg &= ~portInfo[iPort].mask;
   digIO[iPort].result    = result;
   *digIO[iPort].result   = (*portInfo[iPort].port & portInfo[iPort].mask) ? 1 : 0;
}

//------------------------------------------------------------------------------

void digOut_enable(UINT08 iPort, SINT16 *result)
{
   if (0 >= iPort && iPort <= 7) ATD0DIEN |= portInfo[iPort].mask;
   *portInfo[iPort].ddrg |=  portInfo[iPort].mask;
   digIO[iPort].result    = result;
   if (*digIO[iPort].result) *portInfo[iPort].port |=  portInfo[iPort].mask;
   else                      *portInfo[iPort].port &= ~portInfo[iPort].mask;
}

//------------------------------------------------------------------------------

void digIO_update(void)
{
   UINT08 iPort;
   for (iPort = 0; iPort < N_PORTS; iPort++) {
      switch (digIO[iPort].state) {
         case io_OFF: break;
         case io_INPUT:
            *digIO[iPort].result = (*portInfo[iPort].port & portInfo[iPort].mask) ? 1 : 0;
            break;
         case io_OUTPUT:
            if (*digIO[iPort].result) *portInfo[iPort].port |=  portInfo[iPort].mask;
            else                      *portInfo[iPort].port &= ~portInfo[iPort].mask;
            break;
      }
   }
}

//------------------------------------------------------------------------------
