//------------------------------------------------------------------------------
//--  Emulated PWM - GPIO Support                                             --
//--                                                                          --
//--  Copyright (c) 2006 by Eric Fahlgren & Lance Gardiner                    --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#include "hcs12def.h"
#include "ms2def.h"
#include "gpio.h"

#ifndef GCC_BUILD
#  pragma CODE_SEG DEFAULT
#endif

//------------------------------------------------------------------------------

static s_ePWM epwm[N_PORTS];

//------------------------------------------------------------------------------

void ePWM_init(void)
{
   UINT08 iPort;
   for (iPort = 0; iPort < N_PORTS; iPort++) {
      epwm[iPort].enabled = FALSE;
   }
}

//------------------------------------------------------------------------------

void ePWM_enable(UINT08 iPort, UINT16 period, SINT16 *dutycycle)
{
   epwm[iPort].enabled   = TRUE;
   epwm[iPort].isOn      = FALSE;
   epwm[iPort].currentDC = dutycycle;
   epwm[iPort].period    = period;
   epwm[iPort].nextOn    = tix+period;
   epwm[iPort].nextOff   = epwm[iPort].nextOn + *dutycycle * period / 1000;

   *portInfo[iPort].ddrg |=  portInfo[iPort].mask;
// steal port enable logic from digOut_enable(iPort, NULL);
}

//------------------------------------------------------------------------------

void ePWM_update(void)
{
   UINT08 iPort;
   for (iPort = 0; iPort < N_PORTS; iPort++) {
      if (epwm[iPort].enabled) {
         if (epwm[iPort].isOn) {
            if (tix >= epwm[iPort].nextOff) {
               *portInfo[iPort].port &= ~portInfo[iPort].mask;
               epwm[iPort].isOn       =  FALSE;
               epwm[iPort].nextOff    =  epwm[iPort].nextOn + *epwm[iPort].currentDC * epwm[iPort].period / 1000;
            }
         }
         else {
            if (tix >= epwm[iPort].nextOn) {
               *portInfo[iPort].port |=  portInfo[iPort].mask;
               epwm[iPort].isOn       =  TRUE;
               epwm[iPort].nextOn    +=  epwm[iPort].period;
            }
         }
      }
   }
}

//------------------------------------------------------------------------------
