//------------------------------------------------------------------------------
//--  GPIO - General Purpose I/O for CAN-Enabled MegaSquirt                   --
//--                                                                          --
//--  Copyright (c) 2006 by Eric Fahlgren & Lance Gardiner                    --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#ifndef FUNCTION_H
#define FUNCTION_H
//==============================================================================

typedef enum {
   f_or,  f_and, f_xor,
   f_add, f_sub, f_mul, f_div,
   f_operators, // Used as count for above operators.

   f_convert, //  1 arg   r <- x*s/d + t
   f_curve,   //  1 arg   r <- interp(x, curve)
   f_table,   //  2 args  r <- interp(x, y, table)
   f_lt,      //  Relational operators
   f_le,
   f_gt,
   f_ge,
   f_eq,
   f_ne,
} t_Function;

//------------------------------------------------------------------------------
// Conversion

typedef struct {
   UINT08 functionType;
   SINT16 scale;
   SINT16 divid;
   SINT16 trans;
} s_Func_cvt;

//------------------------------------------------------------------------------
// Curve

typedef struct {
   UINT08  functionType;
   UINT08  nPoints;
   SINT16  data[1]; // Actually an arbitrarily long sequence.
} s_Func_crv;

//------------------------------------------------------------------------------
// Table

typedef struct {
   UINT08  functionType;
   UINT08  nX;
   UINT08  nY;
   SINT16  data[1]; // nX of x bins, nY of y, nX*nY of z
} s_Func_tbl;

//------------------------------------------------------------------------------
// Condition

typedef struct {
   UINT08  functionType;
   SINT16  hysteresis;
} s_Func_cnd;

//------------------------------------------------------------------------------
// Booleans and arithmetic have no additional info.

typedef struct {
   union {
      s_Func_cvt s;
      s_Func_crv c;
      s_Func_tbl t;
      s_Func_cnd n;
   } u;
} s_FunctionDefinition;

typedef s_FunctionDefinition *FunctionDefinition;

//------------------------------------------------------------------------------
// FNC_SEG  = READ_ONLY 0x004000 TO 0x0047FF;

#include "funcDef.h"

#define FNCTABLE     ((UINT16 *)0x4000)
#define FNCTABLESIZE sizeof(s_FuncDef)

//------------------------------------------------------------------------------

typedef struct {
   FunctionDefinition  func;
   SINT16             *input1;
   SINT16             *input2;
   SINT16             *output;
} s_Function;

typedef s_Function *Function;

//------------------------------------------------------------------------------

FAR_PROTO(void func_Evaluate(void));
FAR_PROTO(void func_Add     (UINT08 iDef, SINT16 *out, SINT16 *in1, SINT16 *in2));
FAR_PROTO(void func_Init    (UINT08 nFunc));

//==============================================================================
#endif
