/*************************************************************************
**************************************************************************
**   M E G A S Q U I R T  II - 2 0 0 4 - V1.000
**
**   (C) 2003 - B. A. Bowling And A. C. Grippo
**
**   This header must appear on all derivatives of this code.
**
***************************************************************************
**************************************************************************/

/*************************************************************************
**************************************************************************
**   GCC Port
**
**   (C) 2004, 2005 - Philip L Johnson
**
**   This header must appear on all derivatives of this code.
**
***************************************************************************
**************************************************************************/

//------------------------------------------------------------------------------
//--  GPIO - General Purpose I/O for CAN-Enabled MegaSquirt                   --
//--                                                                          --
//--  Copyright (c) 2005, 2006 by Eric Fahlgren & Lance Gardiner              --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------
//--  Version 1.000                                                           --
//--     EF 2005-10-02 - Derived from Version 2.300 of the base MS-II code.   --
//--     EF 2006-01-27 - Merged in 2.500 changes like RAM-based burner.       --
//--                                                                          --
//--                                                                          --
//------------------------------------------------------------------------------

#include "hcs12def.h"      /* common defines and macros */
#include "flash.h"         /* flashburner defines, structures */
#include <string.h>

#include "ms2def.h"
#include "gpio.h"

#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#  pragma DATA_SEG  DEFAULT
#  pragma CONST_SEG DEFAULT
#endif

//==============================================================================
//==  Interrupt vector table setup  ============================================
#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif

INTERRUPT_PROTO(void ISR_Unused       (void));

INTERRUPT_PROTO(void ISR_Clock        (void));
INTERRUPT_PROTO(void ISR_SCI_Comm     (void));
INTERRUPT_PROTO(void ISR_Timer0       (void));
INTERRUPT_PROTO(void ISR_Timer1       (void));
INTERRUPT_PROTO(void ISR_Timer2       (void));
INTERRUPT_PROTO(void ISR_Timer3       (void));
INTERRUPT_PROTO(void ISR_Timer4       (void));
INTERRUPT_PROTO(void ISR_Timer5       (void));
INTERRUPT_PROTO(void ISR_Timer6       (void));
INTERRUPT_PROTO(void ISR_Timer7       (void));
INTERRUPT_PROTO(void ISR_TimerOverflow(void));

typedef void (* NEAR tIsrFunc)(void);
const tIsrFunc _vect[] VECT_ATTR = {
   ISR_Unused,                 /* 63 - Reserved                      */
   ISR_Unused,                 /* 62 - Reserved                      */
   ISR_Unused,                 /* 61 - Reserved                      */
   ISR_Unused,                 /* 60 - Reserved                      */
   ISR_Unused,                 /* 59 - Reserved                      */
   ISR_Unused,                 /* 58 - VREG LVI                      */
   ISR_Unused,                 /* 57 - PWM Emergency Shutdown        */
   ISR_Unused,                 /* 56 - Port P                        */
   ISR_Unused,                 /* 55 - Reserved                      */
   ISR_Unused,                 /* 54 - Reserved                      */
   ISR_Unused,                 /* 53 - Reserved                      */
   ISR_Unused,                 /* 52 - Reserved                      */
   ISR_Unused,                 /* 51 - Reserved                      */
   ISR_Unused,                 /* 50 - Reserved                      */
   ISR_Unused,                 /* 49 - Reserved                      */
   ISR_Unused,                 /* 48 - Reserved                      */
   ISR_Unused,                 /* 47 - Reserved                      */
   ISR_Unused,                 /* 46 - Reserved                      */
   ISR_Unused,                 /* 45 - Reserved                      */
   ISR_Unused,                 /* 44 - Reserved                      */
   ISR_Unused,                 /* 43 - Reserved                      */
   ISR_Unused,                 /* 42 - Reserved                      */
   ISR_Unused,                 /* 41 - Reserved                      */
   ISR_Unused,                 /* 40 - Reserved                      */
   CanTxIsr,                   /* 39 - CAN TX                        */
   CanRxIsr,                   /* 38 - CAN RX                        */
   CanRxIsr,                   /* 37 - CAN Errors                    */
   ISR_Unused,                 /* 36 - CAN Wakeup                    */
   ISR_Unused,                 /* 35 - FLASH                         */
   ISR_Unused,                 /* 34 - Reserved                      */
   ISR_Unused,                 /* 33 - Reserved                      */
   ISR_Unused,                 /* 32 - Reserved                      */
   ISR_Unused,                 /* 31 - Reserved                      */
   ISR_Unused,                 /* 30 - Reserved                      */
   ISR_Unused,                 /* 29 - CRG Self Clock Mode           */
   ISR_Unused,                 /* 28 - CRG PLL Lock                  */
   ISR_Unused,                 /* 27 - Reserved                      */
   ISR_Unused,                 /* 26 - Reserved                      */
   ISR_Unused,                 /* 25 - Reserved                      */
   ISR_Unused,                 /* 24 - Port J                        */
   ISR_Unused,                 /* 23 - Reserved                      */
   ISR_Unused,                 /* 22 - ATD                           */
   ISR_Unused,                 /* 21 - Reserved                      */
   ISR_SCI_Comm,               /* 20 - SCI                           */
   ISR_Unused,                 /* 19 - SPI                           */
   ISR_Unused,                 /* 18 - Pulse Accumulator Input Edge  */
   ISR_Unused,                 /* 17 - Pulse Accumulator A Overflow  */
   ISR_TimerOverflow,          /* 16 - Standard Timer Overflow       */
   ISR_Timer7,                 /* 15 - TC7                           */
   ISR_Timer6,                 /* 14 - TC6                           */
   ISR_Timer5,                 /* 13 - TC5                           */
   ISR_Timer4,                 /* 12 - TC4                           */
   ISR_Timer3,                 /* 11 - TC3                           */
   ISR_Timer2,                 /* 10 - TC2                           */
   ISR_Timer1,                 /* 09 - TC1                           */
   ISR_Timer0,                 /* 08 - TC0                           */
   ISR_Clock,                  /* 07 - RTI                           */
   ISR_Unused,                 /* 06 - !IRQ                          */
   ISR_Unused,                 /* 05 - !XIRQ                         */
   ISR_Unused,                 /* 04 - SWI/BDM                       */
   ISR_Unused,                 /* 03 - Unimplemented Opcode          */
   ISR_Unused,                 /* 02 - COP Reset                     */
   ISR_Unused,                 /* 01 - Monitor Reset                 */
   STARTUP_VECTOR              /* 00 - Reset Vector                  */
};

//==============================================================================
//==  Constants data  ==========================================================

#ifndef GCC_BUILD
#  pragma CONST_SEG INP_ROM
#endif

const s_configuration flashInputs EEPROM_ATTR = {
   2,
   { 0, 0, CAN_GPIO_DEVICE, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},

   // Default configuration.
   { adcIn,        //  0 - PAD00 
     adcIn,        //  1 - PAD01 
     adcIn,        //  2 - PAD02 
     adcIn,        //  3 - PAD03 
     adcIn,        //  4 - PAD04 
     adcIn,        //  5 - PAD05 
     adcIn,        //  6 - PAD06 
     adcIn,        //  7 - PAD07 
     digitalOut,   //  8 - PA0
     digitalOut,   //  9 - PB4
     digitalOut,   // 10 - PE0
     digitalIn,    // 11 - PE1
     digitalOut,   // 12 - PE4
     digitalOut,   // 13 - PM2
     digitalOut,   // 14 - PM3
     digitalOut,   // 15 - PM4
     digitalOut,   // 16 - PM5
     timerCounter, // 17 - PT0
     nativePWM8,   // 18 - PT1
     nativePWM8,   // 19 - PT2
     nativePWM8,   // 20 - PT3
     nativePWM8,   // 21 - PT4
     timerCounter, // 22 - PT5
     timerCounter, // 23 - PT6
     timerCounter  // 24 - PT7
   },

   0x06, // Prescale A, p.358
   0x93, // Scale A, p.364
   0x06, // Prescale B
   0x49, // Scale B

   { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,  // PAD0n
     0xFF, 0xFF, 0xFF, 0xFF, 0xFF,                    // PA/B/E
     0xFF, 0xFF, 0xFF, 0xFF,                          // PMn
     0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,              // PWM8
     0xFF, 0xFF },                                    // PTn
   {    0,    0,    0,    0,    0,    0,    0,    0,  // DCx10
        0,    0,    0,    0,    0,             
        0,    0,    0,    0,                      
      500,  500,  500,  500,  500,  500,
        0,    0
   },

   {    6,    6,    6,    6,    6,    6,    6,    6 }, // Timer ticks per cycle
};

//==============================================================================
//==  All the other constants  =================================================

#ifndef GCC_BUILD
#  pragma CONST_SEG OVF_ROM
#endif

const UINT32 baudRate = 115200;

// array of TC overflow numbers at which to increment secs cntr.
const UINT16 TC_ovfla[60] = {
     23,   46,   69,   92,  114,  137,  160,  183,  206,  229,
    252,  275,  298,  320,  343,  366,  389,  412,  435,  458,
    481,  504,  526,  549,  572,  595,  618,  641,  664,  687,
    710,  732,  755,  778,  801,  824,  847,  870,  893,  916,
    938,  961,  984, 1007, 1030, 1053, 1076, 1099, 1122, 1144,
   1167, 1190, 1213, 1236, 1259, 1282, 1305, 1328, 1350, 1373
};

const char RevNum   [20] = { "MSGPIO Rev 1.00.00" }, // Functional changes/bug fixes.
           Signature[16] = { "MSGPIO Ver 1.00"    }; // Interface changes.

//------------------------------------------------------------------------------

const s_portInfo portInfo[N_PORTS] = {
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  0, 0x01 }, //  0 - PAD00
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  1, 0x02 }, //  1 - PAD01
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  2, 0x04 }, //  2 - PAD02
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  3, 0x08 }, //  3 - PAD03
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  4, 0x10 }, //  4 - PAD04
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  5, 0x20 }, //  5 - PAD05
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  6, 0x40 }, //  6 - PAD06
{              emulatedPWM | digitalOut | digitalIn                | adcIn, pDDRAD, pPTAD,  7, 0x80 }, //  7 - PAD07
{              emulatedPWM | digitalOut | digitalIn                       , pDDRA,  pPORTA, 0, 0x01 }, //  8 - PA0
{              emulatedPWM | digitalOut | digitalIn                       , pDDRB,  pPORTB, 4, 0x10 }, //  9 - PB4
{              emulatedPWM | digitalOut | digitalIn                       , pDDRE,  pPORTE, 0, 0x01 }, // 10 - PE0
{              emulatedPWM | digitalOut | digitalIn                       , pDDRE,  pPORTE, 1, 0x02 }, // 11 - PE1
{              emulatedPWM | digitalOut | digitalIn                       , pDDRE,  pPORTE, 4, 0x10 }, // 12 - PE4
{              emulatedPWM | digitalOut | digitalIn                       , pDDRM,  pPORTM, 2, 0x04 }, // 13 - PM2
{              emulatedPWM | digitalOut | digitalIn                       , pDDRM,  pPORTM, 3, 0x08 }, // 14 - PM3
{              emulatedPWM | digitalOut | digitalIn                       , pDDRM,  pPORTM, 4, 0x10 }, // 15 - PM4
{              emulatedPWM | digitalOut | digitalIn                       , pDDRM,  pPORTM, 5, 0x20 }, // 16 - PM5
{ nativePWM8 | emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 0, 0x01 }, // 17 - PT0
{ nativePWM8 | emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 1, 0x02 }, // 18 - PT1
{ nativePWM8 | emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 2, 0x04 }, // 19 - PT2
{ nativePWM8 | emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 3, 0x08 }, // 20 - PT3
{ nativePWM8 | emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 4, 0x10 }, // 21 - PT4
{ nativePWM8 | emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 5, 0x20 }, // 22 - PT5
{              emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 6, 0x40 }, // 23 - PT6
{              emulatedPWM | digitalOut | digitalIn | timerCounter        , pDDRT,  pPORTT, 7, 0x80 }  // 24 - PT7
};

//==============================================================================

#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#  pragma DATA_SEG  DEFAULT
#  pragma CONST_SEG DEFAULT
#endif

//------------------------------------------------------------------------------
//--  Clocks and counters  -----------------------------------------------------
//
UINT16 mms;
UINT16 millisec;
UINT32 tix;

//------------------------------------------------------------------------------
//--  Flash burning  -----------------------------------------------------------

#define FLASH_LOCKED 0x00
#define FLASH_OPENED 0xCC
#define FLASH_OPEN   ;flashLock=FLASH_OPENED;
#define FLASH_LOCK   ;flashLock=FLASH_LOCKED;

#define NOT_BURNING  0xFFFF

UINT08 flashLock;
UINT08 burnTable;
UINT16 burnIndex;

volatile UINT08 RamBurnPgm[36]; // allocate space in ram for flash burner core

UINT08 reinitialize;

//------------------------------------------------------------------------------
//--  Inputs and outputs  ------------------------------------------------------

s_configuration   ramInputs;
s_variables ramOutputs, frozenOutputs;

//==============================================================================
//==  Define table mapping in ROM, after RAM so we can reference RAM vars.  ====

#ifndef GCC_BUILD
#  pragma CONST_SEG OVF_ROM
#endif

const s_tableDescriptor tables[] =  {
   { (UINT16 *)&ramInputs,     (UINT16 *)&flashInputs,  sizeof(s_configuration)},
   { (UINT16 *)&frozenOutputs, NULL,                    sizeof(frozenOutputs)  },
   { NULL,                     FNCTABLE,                FNCTABLESIZE           },
};

UINT08 nTables(void) { return sizeof(tables)/sizeof(s_tableDescriptor); }

void tableInit(void)
{
   UINT08 iTable;
   for (iTable = 0; iTable < nTables(); iTable++) {
      if (tables[iTable].addrRam != NULL && tables[iTable].addrFlash != NULL) {
         (void)memcpy(tables[iTable].addrRam, tables[iTable].addrFlash, tables[iTable].n_bytes);
      }
   }
}

#define tableByteRam(iTable, iByte)       ((UINT08 *)tables[iTable].addrRam + iByte)
#define tableWordRam(iTable, iWord)       (          tables[iTable].addrRam + iWord)
#define tableByteFlash(iTable, iByte)     ((UINT08 *)tables[iTable].addrFlash + iByte)
#define tableWordFlash(iTable, iWord)     (          tables[iTable].addrFlash + iWord)
#define tableSectorFlash(iTable, iSector) (          tables[iTable].addrFlash + iSector*SECTOR_WORDS)

#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#endif

FAR_PROTO(void   tableInit   (void)         );
FAR_PROTO(UINT16 tableBytes  (UINT08 iTable));
FAR_PROTO(UINT16 tableWords  (UINT08 iTable));
FAR_PROTO(UINT08 tableSectors(UINT08 iTable));

UINT16 tableBytes  (UINT08 iTable) { return tables[iTable].n_bytes; }
UINT16 tableWords  (UINT08 iTable) { return (tables[iTable].n_bytes+1)/2; } // Round up
UINT08 tableSectors(UINT08 iTable) { return (UINT08)((tables[iTable].n_bytes+SECTOR_BYTES-1)/SECTOR_BYTES); }

//==============================================================================

#ifndef GCC_BUILD
#  pragma CONST_SEG DEFAULT
#endif

FAR_PROTO(void main       (void));
FAR_PROTO(void clock_Init (void));

FAR_PROTO(void port_Init  (void));
FAR_PROTO(void sci_Init   (void));
FAR_PROTO(void timer_Init (void));

FAR_PROTO(void flash_Init       (UINT32 oscclk));
FAR_PROTO(void flash_EraseSector(UINT16 *address));
FAR_PROTO(void flash_EraseTable (UINT08 iTable));
FAR_PROTO(void flash_WriteWord  (UINT16 *address, UINT16 data));
FAR_PROTO(void flash_BurnTblData(UINT08 iTable, UINT16 iWord, UINT16 data));
FAR_PROTO(void flash_BurnTblRAM (UINT08 iTable, UINT16 iWord));
FAR_PROTO(void flash_Burn       (void));

FAR_PROTO(extern void reboot (void));
FAR_PROTO(extern void monitor(void));
extern void SpSub(void);
extern void NoOp (void);

//==============================================================================

void main(void)
{
   clock_Init();

   //---------------------------------------------------------------------------
   //--  Set up for flash burning  ---------------------------------------------
   flash_Init(8000); // Enable flash programming capability.

   //---------------------------------------------------------------------------
   // Load all user inputs from Flash to RAM
   tableInit();

   timer_Init();
   sci_Init();

   func_Init(1);
   func_Add(f_add, &ramOutputs.test, ramOutputs.portValue+1, ramOutputs.portValue+2);

   port_Init();

   //---------------------------------------------------------------------------
   // Other slots are edited by user, but...
   ramInputs.canType[ramInputs.canID] = CAN_GPIO_DEVICE;
   Can_Init(ramInputs.canID);

   //---------------------------------------------------------------------------
   // Make IC highest priority interrupt, and enable global interrupts.
   HPRIO = 0xEE;
   ENABLE_INTERRUPTS

   reinitialize = FALSE;
   for ( ; ; )  {
      func_Evaluate();

      digIO_update();
      ePWM_update ();  // Toggle the slow, bit-bang PWM channels.
      pwm_update  ();  // Update the DC on the native ones.

{ // TEST JUNK
   static UINT08 mask     = 0x04;
   static UINT32 nextTime = 0;

   UINT16 nTix = ramOutputs.portValue[2];
   if (tix > nextTime) {
      mask <<= 1;
      if (mask >= 0x40) mask = 0x04;
      PORTM = mask;
      nextTime += nTix;
   }

   if (ramOutputs.seconds % 2)
      PORTE  &= ~0x10;
   else
      PORTE  |=  0x10;
}
      
      //------------------------------------------------------------------------
      //--  Perform flash burning asynchronously.  -----------------------------
      flash_Burn();

      //------------------------------------------------------------------------
      //--  Perform asynchronous CAN operations.  ------------------------------
      if (Can_NeedsReset()) {
         Can_Init(ramInputs.canID);
      }
      Can_Send();

      //------------------------------------------------------------------------
      if (reinitialize) {
         port_Init();
         reinitialize = FALSE;
      }
   }
}

//------------------------------------------------------------------------------

void clock_Init(void)
{
   UINT08 ix;

   //PPAGE = 0x3C;
   // Initialize PLL - Reset default is Oscillator clock 8 MHz
   //    oscillator, PLL freq = 48 MHz, 24 MHz bus, divide by 16 for
   //    timer of 2/3 usec tic
   PLLCTL &= 0xBF; // Turn off PLL so can change freq
   SYNR    = 0x02; // set PLL/ Bus freq to 48/ 24 MHz
   REFDV   = 0x00;
   PLLCTL |= 0x40; // Turn on PLL

   while (!(CRGFLG & 0x08)) ; // wait for PLL lock
   CLKSEL = 0x80;      // select PLL as clock

   for (ix = 0; ix < 60; ix++) ; // wait for clock transition to finish

   // Set up CRG RTI Interrupt for 0.128 ms clock. CRG from 8MHz oscillator.
   RTICTL  = 0x10;  // Load timeout register for 0.128 ms (smallest possible).
   CRGINT |= 0x80;  // Enable interrupt.
   CRGFLG  = 0x80;  // Clear interrupt flag (0 writes have no effect).
}

//==============================================================================
//==  GPO Primitives  ==========================================================

typedef struct {
   UINT08 *port;
   UINT08  mask;
   UINT08  var[2];
   UINT16  dir[2];
   UINT16  lo[2];
   UINT16  hi[2];
} s_gpo;

typedef s_gpo *gpo;

void gpo_Create(gpo p, UINT08 *portId, UINT08 bitNum)
{
   p->port = portId;
   p->mask = 0x01 << bitNum;
}

//==============================================================================

void timer_Init(void)
{
   TTOV   = 0x00;
   TSCR1  = 0x00;   // Disable all timers.

// TIOS  |= 0x3E;   // Timer ch 0 = IC, ch 1-5 = OC & PWM, ch 6,7 = I/O output.
         // 0011 1110

 //TIOS   = 0x2A;   // Timer ch 0 = IC, ch 1-5 = OC & PWM, ch 6,7 = I/O output.
         // 0010 1010
// TIOS = 0x0A;
// TIOS = 0x14;

   // Set prescaler to 16. This divides bus clk (= PLLCLK/2 = 24 MHz)
   //   by 16. This gives 1.5 MHz timer, 1 tic = 2/3 us.
   TSCR2  = 0x04;
// TCTL2 |= 0x88;   // Bit OM1,3,5 = 1. OC output line high or low iaw OL1,3,5 (not toggle or disable).
// TCTL1 |= 0x08;

   TSCR1  = 0x80; // Turn on main timer.
   TSCR2 |= 0x80;   // Enable timer overflow interrupt (TOI)

   //---------------------------------------------------------------------------
   mms                = 0; // 0.128 ms tix
   tix                = 0;

   millisec           = 0; // 1.024 ms clock (8 tix) for adcs
   ramOutputs.ticks   = 0; // 1.024 ms
   ramOutputs.seconds = 0;
}

//==============================================================================
//==  Overall Port Control  ====================================================
//
// The general purpose input output board default port assignments are:
//
//    * ADC port (PAD00 - PAD07) to the general purpose inputs (GPIx) and
//      the exhaust gas temperature (EGTx) input circuits,
//
//    * Timer port (PT0 - PT7) to the pulse width modulation outputs (PWMx)
//      and the variable reluctor (VRx) input circuits,
//
//    * The general purpose output (GPOx) circuits and the high current
//      driver (VBx) circuits are on various PAx, PBx, PEx and PMx ports.
//
//------------------------------------------------------------------------------

void port_Init(void)
{
   UINT08 iPort;
   UINT08 iBlock;

   // Set all unused (even unbonded) ports to inputs with pullups.
   DDRA    = 0x01;   // port A0 - output
   DDRB    = 0x10;   // port B4 - output
   DDRE    = 0x13;   // port E4 - output, E0 is input
   PUCR   |= 0x13;   // enable pullups for ports E, B and A
   DDRM    = 0xFF;   // port M  - all outputs, full drive by default
   DDRP    = 0x00;
   PERP    = 0xFF;   // enable pullup resistance for port P
   DDRJ    = 0x3F;
   PERJ   |= 0xC0;   // enable pullup resistance for port J6, 7
   DDRT    = 0xFF;

   adc_init();
   digIO_init();
   ePWM_init();
   pwm_init(ramInputs.prescaleA, ramInputs.scaleA, ramInputs.prescaleB, ramInputs.scaleB);
   inputCapture_init();

   for (iPort = 0; iPort < N_PORTS; iPort++) {
      if (ramInputs.portUsage[iPort] & portInfo[iPort].validUse) {
         switch (ramInputs.portUsage[iPort]) {
            case unused:
               break;
            case nativePWM8:
               iBlock = iPort-17;
               ramOutputs.portValue[iPort] = ramInputs.initDC[iPort];
               pwm_enable(iBlock, ramInputs.period[iPort], ramOutputs.portValue+iPort);
               break;
            case emulatedPWM:
               ePWM_enable(iPort, ramInputs.period[iPort], ramOutputs.portValue+iPort);
               break;
            case digitalOut:
               digOut_enable(iPort, ramOutputs.portValue+iPort);
               break;
            case digitalIn:
               digInp_enable(iPort, ramOutputs.portValue+iPort);
               break;
            case timerCounter:
               iBlock = iPort-17;
               inputCapture_enable(iBlock, ramInputs.tixPerCycle[iBlock], ramOutputs.portValue+iPort, ramOutputs.count+iBlock);
               break;
            case adcIn:
               adc_enable(iPort, ramOutputs.portValue+iPort);
               break;
         }
      }
   }

// MS-II injector PWMs - PWM2, 4, if you want to test them
TCTL2 |=  0xCC; // TEST JUNK set all bits on MS-II OL1,3, get rid of this for real GPIO board
   adc_update();
}

//------------------------------------------------------------------------------
#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif

INTERRUPT_FUNC void ISR_Clock(void)
{
   // 0.128 ms clock interrupt - clear flag immediately to resume count
   CRGFLG = 0x80;  // Clear RTI interrupt flag

   // Also generate 1.024 ms, 0.10035 sec and 1.0035 sec clocks
   tix++;   // Free running clock(0.128 ms tix) good for ~110 hrs
   mms++;   // in 0.128 ms tix - reset every 8 tix = 1.024 ms

   // Check mms to generate other clocks
   if (mms >= 8) {
      ramOutputs.ticks++; // Running counter
      mms = 0;
      millisec++;         // Actually 1.024 ms
      if (millisec >= 98) millisec = 0;

      // Update ADC values every ms.
      // ????????????? OR WHATEVER, add sync events to adc objects?
      adc_update();
   }

   inputCapture_debounce();
}

//------------------------------------------------------------------------------
#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif

INTERRUPT_FUNC void ISR_TimerOverflow(void)
{
   static UINT16 TC_ovflow = 0;
   static UINT16 TC_ov_ix  = 0;

   // Get display seconds from continuously running TCNT.
   TC_ovflow++;
   if (TC_ovflow >= TC_ovfla[TC_ov_ix]) {
      // Update "accurate" seconds.
      ramOutputs.seconds++;
      TC_ov_ix++;
      if (TC_ov_ix > 59) {
         TC_ov_ix  = 0;
         TC_ovflow = 0;
      }
   }

   // Clear timer overflow interrupt flag (TOF).
   TFLG2  = 0x80;
}

INTERRUPT_FUNC void ISR_Timer0(void) { inputCapture_capture(0); }
INTERRUPT_FUNC void ISR_Timer1(void) { inputCapture_capture(1); }
INTERRUPT_FUNC void ISR_Timer2(void) { inputCapture_capture(2); }
INTERRUPT_FUNC void ISR_Timer3(void) { inputCapture_capture(3); }
INTERRUPT_FUNC void ISR_Timer4(void) { inputCapture_capture(4); }
INTERRUPT_FUNC void ISR_Timer5(void) { inputCapture_capture(5); }
INTERRUPT_FUNC void ISR_Timer6(void) { inputCapture_capture(6); }
INTERRUPT_FUNC void ISR_Timer7(void) { inputCapture_capture(7); }

//------------------------------------------------------------------------------
//--  Set up SCI (rs232): SCI BR reg= BusFreq(=24MHz)/16/baudrate             --

#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#endif

void sci_Init(void)
{
   UINT32 baudRemainder;

   SCI0BDL = (UINT08)(1500000/baudRate);
   baudRemainder = (150000000/baudRate) - ((UINT32)SCI0BDL*100);
   if (baudRemainder > 50) SCI0BDL++;   // round up
   SCI0CR1       = 0x00;
   SCI0CR2       = 0x24;    // TIE=0, RIE = 1; TE=0, RE =1

   DDRS         &= 0xF3;    // SCI port setup.
   PERS         |= 0x0C;    // Enable pullup resistance for port S2, 3
}

//------------------------------------------------------------------------------
// SCI Communications
//
// Communications is established when the PC communications program sends
// a command character - the particular character sets the mode:
//
// a<can><tbl>
//    send all of the realtime display variables (ramOutputs structure) via txport.
//
// w<can><tbl><ofs-msb><ofs-lsb><len-msb><len-lsb><n-data-bytes>
//    receive updated data parameter(s) and write into offset location
//    relative to start of data block
//
// e
//    same as "w" above, followed by "r" below to echo back value
//
// r<can><tbl><ofs-msb><ofs-lsb><len-msb><len-lsb>
//    read and send back value of a data parameter or block in offset location
// y
//    verify ramInputs data block = inpflash data block, return no. bytes different.
//
// b<can><tbl>
//    jump to flash burner routine and burn a ram data block into a flash
//    data block.
//
// t<tbl-id><table-bytes>
//    Update the given table with the bytes.  Direct to flash programming
//    is performed.  After you send the first two characters (the "t" command
//    and the table id byte), you must wait sufficient time for the erase
//    routines to complete, 100 ms is more than sufficient.
//
// !x
//    Reintialize
// !!x
//    Reboot
// !!!
//    Enter monitor mode
//
// c
//    Test communications - echo back current ramOutputs.seconds
//
// Q
//    Send over Embedded Code Revision Number
//
// S
//    Send program title.
//
//------------------------------------------------------------------------------

#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif

INTERRUPT_FUNC void ISR_SCI_Comm(void)
{
   UINT08 dummy;
   UINT16 ix;

   static UINT16 iTblWord    = 0;
   static UINT16 tble_word   = 0;

   static UINT08 verifyFailures;
   static UINT08 rwMode;
   static UINT08 CANid, ibuf;
   static UINT08 tableId;
   static UINT08 pendingTXmode;
   static UINT08 txmode      = 0;
   static UINT16 txcnt       = 0;
   static UINT16 rxcnt       = 0;
   static UINT16 txgoal      = 0;
   static UINT16 rxoffset    = 0;
   static UINT16 rxnbytes    = 0;
   static UINT32 recvTimeout = 0xFFFFFFFF;

#define RW_READ  0
#define RW_WRITE 1
#define RW_ECHO  2

#define getCANid             40
#define getTableId           41
#define setBurningParameters 99

   //---------------------------------------------------------------------------
   if ((SCI0SR1 & 0x20)) { // if RDRF register not set, => transmit interrupt
      // Receive Interrupt
      //   Clear the RDRF bit by reading SCISR1 register (done above),
      //   then read data (in SCIDRL reg).

      txgoal      = 0;
      recvTimeout = 0xFFFFFFFF;

      switch (txmode) {
         case 0: //--  COMMAND INPUT  ------------------------------------------
            switch(SCI0DRL)  { //-----------------------------------------------
               case 'a': // send back all real time ram output variables
                  pendingTXmode = 1;
                  txmode        = getCANid;
                  break;

               case 'w':        // receive new ram input data and write into offset location
                  pendingTXmode = 5;
                  txmode        = getCANid;
                  rwMode        = RW_WRITE;
                  break;

               case 'e':        // same as 'w', but verify by echoing back value
                  pendingTXmode = 5;
                  txmode        = getCANid;
                  rwMode        = RW_ECHO;
                  break;

               case 'r':        // read and send back ram input data from offset location
                  pendingTXmode = 5;
                  txmode        = getCANid;
                  rwMode        = RW_READ;
                  break;

               case 'y':        // Verify that a flash data block matches a
                  pendingTXmode = 2;        // corresponding ram data block
                  txmode        = getCANid;
                  break;

               case 'b':        // burn a block of ram input values into flash
                  pendingTXmode = setBurningParameters;
                  txmode        = getCANid;
                  break;

               case 't':        // update a flash table with following serial data
                  txmode = 20;
                  break;

               case '!':        // start receiving reinitialize/reboot command
                  txmode = 30;
                  break;

               case 'c':        // send back seconds to test comms
                  txcnt    = 0;
                  txmode   = 1;
                  txgoal   = 2; // seconds is 1st 2 bytes of ramOutputs structure
                  frozenOutputs.seconds = ramOutputs.seconds;
                  SCI0DRL  = *(char *)&frozenOutputs;
                  SCI0CR2 |= 0x88;    // xmit enable & xmit interrupt enable
                  break;

               case 'Q': // send code rev no.
                  txcnt    =  0;
                  txmode   =  4;
                  txgoal   =  20;
                  SCI0DRL  =  RevNum[0];
                  SCI0CR2 |= 0x88;      // xmit enable & xmit interrupt enable
                  break;

               case 'S': // send program title
                  txcnt    =  0;
                  txmode   =  5;
                  txgoal   =  sizeof(Signature)-1;
                  SCI0DRL  =  Signature[0];
                  SCI0CR2 |= 0x88;         // xmit enable & xmit interrupt enable
                  break;

               default:
                  break;
            }
            break;

         //---------------------------------------------------------------------
         case getCANid: //--  Get CAN id for current command.  -----------------
            CANid = SCI0DRL;
               if (CANid < CAN_N_DEVICES && ramInputs.canType[CANid] != 0)
                  txmode = getTableId;
               else
                  txmode = 0;
               break;

         //---------------------------------------------------------------------
         case getTableId: //--  Get table id for current command.  -------------
            tableId = SCI0DRL;
            if (CANid != ramInputs.canID || tableId >= nTables())
               txmode = 0;
            else
               txmode = pendingTXmode;

            pendingTXmode = 0;
            if (txmode == 1) {
               txcnt    = 0;
               txgoal   = tableBytes(tableId);
               // load all output variables into frozenOutputs to avoid incoherent word data
               (void)memcpy(tableByteRam(tableId, 0), &ramOutputs, sizeof(ramOutputs));
               SCI0DRL  = *tableByteRam(tableId, 0);
               SCI0CR2 |= 0x88;    // xmit enable & xmit interrupt enable
            }
            else if (txmode == setBurningParameters && flashLock == FLASH_LOCKED) {
               burnTable   = tableId;
               txmode      = 0;
               burnIndex   = 0;
               FLASH_OPEN;
            }
            else if(txmode == 2)  {
               verifyFailures = 0;
               for(ix = 0; ix < tables[tableId].n_bytes; ix++)  {
                  if(*tableByteFlash(tableId, ix) != *tableByteRam(tableId, ix))
                  verifyFailures++;
               }
               txcnt = 0;
               txgoal = 2;
               SCI0DRL = *(char *)&verifyFailures;
               SCI0CR2 |= 0x88;    // xmit enable & xmit interrupt enable
            }
            break;

         //---------------------------------------------------------------------
         case 5:
            rxoffset  = (SCI0DRL << 8); // byte offset(msb) from start of table
            txmode++;
            break;

         //---------------------------------------------------------------------
         case 6:
            rxoffset  |= SCI0DRL; // byte offset(lsb) from start of table
            txmode++;
            break;

         //---------------------------------------------------------------------
         case 7:
            rxnbytes  = (SCI0DRL << 8); // no. bytes (msb)
            txmode++;
            break;

         //---------------------------------------------------------------------
         case 8:
            rxnbytes |= SCI0DRL; // no. bytes (lsb)
            rxcnt    =  0;
            if (rwMode == RW_READ) { // read & send back input data
               txcnt    = 0;
               txmode   = 3;
               txgoal   = rxnbytes;
               if (tableByteRam(tableId, 0) == 0)
                  SCI0DRL  = *tableByteFlash(tableId, rxoffset);
               else
                  SCI0DRL  = *tableByteRam  (tableId, rxoffset);
               SCI0CR2 |= 0x88; // xmit enable & xmit interrupt enable
            }
            else  { // write data to input data block
               // buffer to maintain coherence of inputs while awaiting serial bytes
               if((rxnbytes > 1) && (rxnbytes < sizeof(frozenOutputs)))
                  ibuf = 1;
               else
                  ibuf = 0;
               txmode++;
            }
            break;

         //---------------------------------------------------------------------
         case 9:
            // Check for data overrun.  Note: this
            // still not bullet proof because could write half the bytes
            // then have overrun; need to store and write all at once if
            // all is valid.
            if ((SCI0SR1 & 0x08) == 0) { // none yet-write data.
               if(!ibuf)
                  *tableByteRam(tableId, rxoffset + rxcnt) = SCI0DRL;
               else
                  *((char *)&frozenOutputs + rxcnt) = SCI0DRL;
            }
            rxcnt++;
            if(rxcnt >= rxnbytes)  {
               if(ibuf)
                  (void)memcpy(tableByteRam(tableId, rxoffset), (char *)&frozenOutputs, rxnbytes);
               dummy = SCI0DRL;      // clear OR bit (read sci0sr1(done), sci0drl)
               if(rwMode == RW_WRITE)
                  txmode = 0;         // done receiving/writing all the data
               else  {
                  txcnt = 0;            // send back data just written
                  txmode = 3;            //    for verification
                  txgoal = rxnbytes;
                  SCI0DRL = *tableByteRam(tableId, rxoffset);
                  SCI0CR2 |= 0x88;    // xmit enable & xmit interrupt enable
               }
            }
            break;

         //---------------------------------------------------------------------
         case 20: //-- Burn new table ------------------------------------------
            tableId = SCI0DRL;
            if (tableId >= nTables())
               txmode = 0;
            else {
               txmode++;
               FLASH_OPEN;
                  flash_EraseTable(tableId);
               FLASH_LOCK;
               iTblWord = 0;
            }
            break;

         //---------------------------------------------------------------------
         case 21:
            tble_word = (SCI0DRL << 8);    // msb
            txmode++;
            break;

         //---------------------------------------------------------------------
         case 22:
            tble_word |= SCI0DRL;          // lsb
            FLASH_OPEN;
               flash_BurnTblData(tableId, iTblWord, tble_word);
            FLASH_LOCK;
            iTblWord++;
            if(iTblWord < tableWords(tableId))
               txmode = 21;
            else
               txmode = 0;
            break;

         //---------------------------------------------------------------------
         case 30:
            if (SCI0DRL == '!') // receive 2nd ! for reboot signal
               txmode++;
            else if (SCI0DRL == 'x') { // received reinitialize signal (!x)
               reinitialize = TRUE;
               txmode = 0;
            }
            else
               txmode = 0;
            break;

         //---------------------------------------------------------------------
         case 31:
            if(SCI0DRL == 'x')  {   // received complete reboot signal(!!x)
               (void)reboot();
            }
            else if(SCI0DRL == '!')  {   // received complete reload signal(!!!)
               SCI0CR1 = 0x00;
               SCI0CR2 = 0x00;      // disable Tx, Rx
               (void)monitor();
            }
            txmode = 0;
            break;

         //---------------------------------------------------------------------
         default: //--  Unknown input  -----------------------------------------
            dummy = SCI0DRL; // Simply read to clear register.
            break;
      }

      if(txgoal == 0 && txmode > 0) {
         recvTimeout = tix + 3906;   // 0.5 sec timeout
      }
   }

   //---------------------------------------------------------------------------
   else { // Transmit Interrupt
      // Clear the TDRE bit by reading SCISR1 register (done), then write data
      txcnt++;
      if(txcnt >= txgoal)  {
         txcnt    = 0;
         txgoal   = 0;
         txmode   = 0;
         SCI0CR2 &= ~0x88; // xmit disable & xmit interrupt disable
      }
      else  {
         switch (txmode) {
            case 3:
               if (tableByteRam(tableId, 0) == 0)
                  SCI0DRL = *tableByteFlash(tableId, rxoffset + txcnt);
               else
                  SCI0DRL = *tableByteRam  (tableId, rxoffset + txcnt);
               break;
            case 4:
               SCI0DRL = RevNum[txcnt];
               break;
            case 5:
               SCI0DRL = Signature[txcnt];
               break;
            case 2:
               SCI0DRL = *((char *)&verifyFailures + txcnt);
               break;
            default:
               SCI0DRL = *tableByteRam(tableId, txcnt);
               break;
         }
      }
   }
}
#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#endif

//------------------------------------------------------------------------------

#ifndef GCC_BUILD
#  pragma CODE_SEG  NON_BANKED
#endif
INTERRUPT_FUNC void ISR_Unused(void)
{
   /* Unimplemented ISRs trap.*/
   return;
}
#ifndef GCC_BUILD
#  pragma CODE_SEG  DEFAULT
#endif

//------------------------------------------------------------------------------
//--  Application specific FLASH burning  --------------------------------------

void flash_EraseTable(UINT08 iTable)
{
   if (flashLock == FLASH_OPENED) {
      // Erase all sectors in segment.
      UINT08 iSector;
      UINT08 nSectors = tableSectors(iTable);
      for (iSector = 0; iSector < nSectors; iSector++) {
         flash_EraseSector(tableSectorFlash(iTable, iSector));
      }
   }
}

void flash_BurnTblRAM(UINT08 iTable, UINT16 iWord)
{
   if (flashLock == FLASH_OPENED) {
      flash_WriteWord(tableWordFlash(iTable, iWord), *tableWordRam(iTable, iWord));
   }
}

void flash_BurnTblData(UINT08 iTable, UINT16 iWord, UINT16 data)
{
   if (flashLock == FLASH_OPENED) {
      flash_WriteWord(tableWordFlash(iTable, iWord), data);
   }
}

void flash_Burn(void)
{
   if (burnIndex < NOT_BURNING) {
      if (burnIndex >= tableWords(burnTable)) {
         burnIndex =  NOT_BURNING;
         FLASH_LOCK;
      }
      else {
         if (burnIndex == 0) flash_EraseTable(burnTable);
         flash_BurnTblRAM(burnTable, burnIndex);
         burnIndex++;
      }
   }
}

//------------------------------------------------------------------------------
//--  Generic FLASH Burning  ---------------------------------------------------
//------------------------------------------------------------------------------
//--  flash_Init                                                              --
//--     Initialize Flash NVM for HCS12 by programming FCLKDIV based on       --
//--     passed oscillator frequency, then uprotect the array, and finally    --
//--     ensure PVIOL and ACCERR are cleared by writing to them.              --
//------------------------------------------------------------------------------

void flash_Init(UINT32 oscclk)
{
   UINT08  fclk_val;

   // Next, initialize FCLKDIV register to ensure we can program/erase.
   if (oscclk >= 12000) {
      fclk_val = oscclk / 8 / 200 - 1; // FDIV8 set since above 12MHz clock.
      FCLKDIV = FCLKDIV | fclk_val | FDIV8;
   }
   else {
      fclk_val = oscclk / 8 / 200 - 1;
      FCLKDIV = FCLKDIV | fclk_val;
   }

   FPROT = 0xFF; // Disable all protection (only in special modes).
   FSTAT = FSTAT | (PVIOL | ACCERR); // Clear any errors.

   if ((int)RamBurnPgm & 0x0001) { // odd address - cpy to even one
      (void)memcpy((void *) RamBurnPgm,    NoOp,   1);   // cpy noop to 1st location
      (void)memcpy((void *)(RamBurnPgm+1), SpSub, 32);   // cpy flashburn core pgm to ram
   }
   else {
      (void)memcpy((void *) RamBurnPgm,    SpSub, 32);   // cpy flashburn core pgm to ram
   }

   FLASH_LOCK;
   burnIndex = NOT_BURNING;
}

void flash_EraseSector(UINT16 *address)
{
   if (flashLock == FLASH_OPENED) {
      FSTAT = (ACCERR | PVIOL);  // Clear errors.
      *address = 0xFFFF;         // Dummy store to page to be erased.
      FCMD = ERASE;
      asm {
        ldaa  #CBEIF
        jsr  RamBurnPgm
      };
   }
}

void flash_WriteWord(UINT16 *address, UINT16 data)
{
   if (flashLock == FLASH_OPENED) {
      FSTAT = (ACCERR | PVIOL);  // Clear errors.
      *address = data;           // Store desired data to address being programmed.
      FCMD = PROG;
      asm {
        ldaa  #CBEIF
        jsr  RamBurnPgm
      };
   }
}

//------------------------------------------------------------------------------
