//------------------------------------------------------------------------------
//--  GPIO - General Purpose I/O for CAN-Enabled MegaSquirt                   --
//--                                                                          --
//--  Copyright (c) 2005, 2006 by Eric Fahlgren & Lance Gardiner              --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#ifndef GPIO_H
#define GPIO_H
//==============================================================================

#include "adc.h"
#include "can_io.h"
#include "digIO.h"
#include "epwm.h"
#include "function.h"
#include "inputCapture.h"
#include "pwm.h"

//------------------------------------------------------------------------------

#define N_PORTS 25

typedef enum {
   unused       = 0x00,
   nativePWM8   = 0x01,
   emulatedPWM  = 0x02,
   digitalOut   = 0x04,
   digitalIn    = 0x08,
   timerCounter = 0x10,
   adcIn        = 0x20
} t_portUsage;


typedef struct {
   UINT08  canID;                  // CAN ID of this device.
   UINT08  canType[CAN_N_DEVICES]; // Board type (1-255) of ith board;

   // Overall port usage settings.
   UINT08  portUsage  [N_PORTS];

   //---------------------------------------------------------------------------
   // Configuration for native PWM8 ports.
   UINT08  prescaleA, scaleA;
   UINT08  prescaleB, scaleB;

   // Period and initDC are used for both PWM8 and emulated PWM,
   // hence N_PORTS rather than N_PWMS.
   UINT08  period[N_PORTS]; // PWM8 = scaled value; emulated PWM = tix.
   UINT16  initDC[N_PORTS]; // %*10, 1000 = 100%

   //---------------------------------------------------------------------------
   // Configuration for Emulated PWM ports.

   //---------------------------------------------------------------------------
   // Configuration for Digital Out ports.

   //---------------------------------------------------------------------------
   // Configuration for Digital In ports.

   //---------------------------------------------------------------------------
   // Configuration for Timer/Counter ports.
   UINT16  tixPerCycle[N_TIMERS];

   //---------------------------------------------------------------------------
   // Configuration for ADC ports.

   //---------------------------------------------------------------------------
   // Function configuration.
#  include "funcDefUsr.con"

} s_configuration;

typedef struct {
   UINT32 ticks;               // Running "msec" clock in 1.024 ms
   UINT16 seconds;             // Running second clock, more accurate than ticks.
   UINT16 count    [N_TIMERS]; // Timer counter accumulators.
   SINT16 portValue[N_PORTS ]; // Raw port values.

   SINT16 test;
   //---------------------------------------------------------------------------
   // Function outputs
#  include "funcDefUsr.var"

} s_variables;

typedef struct {
   UINT16 *addrRam;
   UINT16 *addrFlash;
   UINT16  n_bytes;
} s_tableDescriptor;

//------------------------------------------------------------------------------

typedef struct {
            UINT08  validUse;
   volatile UINT08 *ddrg;
   volatile UINT08 *port;
            UINT08  bitn;
            UINT08  mask;
} s_portInfo;

//------------------------------------------------------------------------------

#ifndef GCC_BUILD
#  pragma CONST_SEG OVF_ROM
#endif

extern       UINT32     tix;
extern const s_portInfo portInfo[N_PORTS];

//==============================================================================
#endif
