//------------------------------------------------------------------------------
//--  MS-II - General Definitions                                             --
//--                                                                          --
//--  Copyright (c) 2005, 2006 by Eric Fahlgren & Lance Gardiner              --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#ifndef MS2DEF_H
#define MS2DEF_H 1

#include "hcs12def.h"      /* common defines and macros */
#include <stdtypes.h>

//------------------------------------------------------------------------------

#define SECTOR_BYTES         1024 // bytes, C32 is 512, C64+ are 1024
#define SECTOR_WORDS         ((int)(SECTOR_BYTES/2))
#define N_SECTORS(theStruct) ((int)((sizeof(theStruct)+SECTOR_BYTES-1)/SECTOR_BYTES))
#define N_PADDING(theStruct) ((int)(N_SECTORS(theStruct)*SECTOR_BYTES - sizeof(theStruct)))

typedef unsigned char  UINT08;
typedef unsigned int   UINT16;
typedef unsigned long  UINT32;
typedef   signed char  SINT08;
typedef   signed int   SINT16;
typedef   signed long  SINT32;

//------------------------------------------------------------------------------

#define MS0010         78        // 7.8125 ticks per ms; 0.128 ms/tick
#define MS0100        781
#define MS1000       7812

//------------------------------------------------------------------------------

#ifdef GCC_BUILD
#  define ENABLE_INTERRUPTS     __asm__ __volatile__ ("cli");
#  define DISABLE_INTERRUPTS    __asm__ __volatile__ ("sei");

#  define INTERRUPT_PROTO(decl) decl __attribute__((interrupt))
#  define INTERRUPT_FUNC        /*nada*/

#  define FAR_PROTO(decl)       decl __attribute__ ((far)) __attribute__ ((section (".text1")))

#  define ADCX_ATTR             __attribute__ ???
#  define EEPROM_ATTR           __attribute__ ((section(".eeprom")))
#  define VECT_ATTR             __attribute__ ((section(".vectors")))
#  define NEAR

   extern void _start(void); /* Startup routine */
#  define STARTUP_VECTOR _start

#else // CW build
#  define ENABLE_INTERRUPTS     asm cli;
#  define DISABLE_INTERRUPTS    asm sei;

#  define INTERRUPT_PROTO(decl) interrupt decl
#  define INTERRUPT_FUNC        interrupt

#  define FAR_PROTO(decl)       decl

#  define ADCX_ATTR             @0x4000
#  define EEPROM_ATTR           /*nada*/
#  define VECT_ATTR             @0xFF80
#  define NEAR                  near

   extern void near _Startup(void); /* Startup routine */
#  define STARTUP_VECTOR _Startup

#endif

//------------------------------------------------------------------------------
#endif
