//------------------------------------------------------------------------------
//--  PWM - GPIO Support                                                      --
//--                                                                          --
//--  Copyright (c) 2006 by Eric Fahlgren & Lance Gardiner                    --
//--  All Rights Reserved.                                                    --
//--                                                                          --
//--  This program is free software; you can redistribute it and/or           --
//--  modify it under the terms of the GNU General Public License             --
//--  as published by the Free Software Foundation; either version            --
//--  2 of the License, or (at your option) any later version.                --
//--  See http://www.gnu.org/licenses/gpl.txt                                 --
//------------------------------------------------------------------------------

#include "hcs12def.h"
#include "ms2def.h"
#include "pwm.h"

#ifndef GCC_BUILD
#  pragma CODE_SEG DEFAULT
#endif

//------------------------------------------------------------------------------

extern UINT32 tix;

static s_PWM pwm[N_PWMS];

//------------------------------------------------------------------------------

void pwm_init(UINT08 prescaleA, UINT08 scaleA, UINT08 prescaleB, UINT08 scaleB)
{
   UINT08 iChannel;
   for (iChannel = 0; iChannel < N_PWMS; iChannel++) {
      pwm[iChannel].enabled = FALSE;
   }

   //---------------------------------------------------------------------------
   //--  Set up timers  --------------------------------------------------------
   //--  Prescale p.358, scale p.364
   PWMPRCLK        &= 0xF0;
   PWMPRCLK        |= prescaleA;
   PWMSCLA          = scaleA;

   PWMPRCLK        &= 0x0F;
   PWMPRCLK        |= prescaleB << 4;
   PWMSCLB          = scaleB;

   PWMCAE           = 0x00;       // Standard left align pulse.
}

//------------------------------------------------------------------------------
//--  PWM 0 1 . . 4 5 are on clock A/SA
//--      . . 2 3 . . are on clock B/SB

void pwm_enable(UINT08 iChannel, UINT08 period, SINT16 *dutyCycle)
{
   UINT08 mask = 1 << iChannel;

   pwm[iChannel].enabled   = TRUE;
   pwm[iChannel].currentDC = dutyCycle;

   MODRR  |=  mask;
   DDRT   |=  mask;
   PORTT  |=  mask;
   TIE    &= ~mask;
   TIOS   &= ~mask;

   PWMPOL |=  mask; // Polarity = 1, i.e., go high on start.
   PWMCLK |=  mask; // Enable scaled clock

   PWMPER[iChannel] = period;
   PWMDTY[iChannel] = (UINT08)(*dutyCycle * (UINT16)PWMPER[iChannel] / 1000);

   PWME  |= mask;
}

//------------------------------------------------------------------------------

void pwm_update(void)
{
   UINT08 iChannel;
   for (iChannel = 0; iChannel < N_PWMS; iChannel++) {
      if (pwm[iChannel].enabled) {
         PWMDTY[iChannel] = (UINT08)(*pwm[iChannel].currentDC * (UINT16)PWMPER[iChannel] / 1000);
      }
   }
}

//------------------------------------------------------------------------------
